@@ -155,8 +155,8 @@ static void process_buddy_icon(FacebookAccount *fba, PurpleBuddy *buddy,
 			/* bigger icon at /profile6/1845/74/n800753867_2878.jpg */
 			search_tmp = strstr(icon_path, "/q");
 			if (search_tmp)
-				*(search_tmp + 1) = 'n';
-				
+				*(search_tmp + 1) = 'n';			
+			purple_debug_info("facebook", "buddy %s has a new buddy icon at http://%s%s\n", buddy->name, icon_host, icon_path);
 			/* Fetch their icon */
 			fb_post_or_get(fba, FB_METHOD_GET, icon_host,
 					icon_path, NULL,
@@ -362,6 +362,8 @@ static void got_status_stream_cb(FacebookAccount *fba, gchar *data,
 			fbuddy = buddy->proto_data;
 			g_free(fbuddy->status);
 			
+			fbuddy->status = purple_strreplace(message_string, "&hearts;", "♥");
+			g_free(message_string); message_string = fbuddy->status;
 			fbuddy->status = purple_markup_strip_html(message_string);
 			
 			purple_prpl_got_user_status(fba->account, buddy->name,
